@@ -552,9 +552,23 @@ it can."
 ;; langauge-info-alist update.
 ;;
 
-(set-language-info "Japanese" 'coding-priority 
-                   '(iso-2022-jp-3-compatible  euc-jisx0213 
-                     japanese-shift-jisx0213 iso-2022-jp-2))
+(set-language-info "Japanese" 'coding-priority
+		   (let ((cand
+			  '(iso-2022-jp-3-compatible
+			    utf-8 utf-16-le utf-16-be
+			    euc-jisx0213 japanese-shift-jisx0213
+			    iso-2022-jp-2))
+			 cs catlist result)
+		     (while cand
+		       (setq cs (car cand)
+			     cand (cdr cand))
+		       (if (and (coding-system-p cs)
+				(not (memq (coding-system-category cs)
+					   catlist)))
+			   (setq result (cons cs result)
+				 catlist (cons (coding-system-category cs)
+					       catlist))))
+		     (nreverse result)))
 
 (coding-system-put 'japanese-shift-jisx0213 'coding-category
                    'coding-category-sjis)
