@@ -27,20 +27,22 @@
 (require 'mucs-ccl)
 (require 'mucs-type)
 
-;;; Type definitions
+(defvar mccl-font-current-charset nil
+  "This variable indicates the charset of current MUCS-CCL environment,
+which is bound by mccl-font-encoder.")
 
-(defconst mccl-font-charset-alist
-  (mapcar
-   (lambda (charset)
-     (let ((typesym
-	    (intern (format "font-%s"
-			    charset))))
-     (mucs-define-type
-      typesym
-      'identity
-      'identity)
-     (cons typesym charset)))
-   (charset-list)))
+;;; conversion type check
+
+(defsubst mccl-font-check-conversion-type ()
+  (if (not (eq (mucs-conversion-get-conv-type
+		mucs-current-conversion)
+	       'font))
+      (signal 'mucs-conversion-type-mismatch-error
+	      (mucs-conversion-get-conv-type
+		mucs-current-conversion)
+	      'font)))
+
+;;; Type definitions
 
 (mucs-define-type
  'font-codepoint
@@ -49,7 +51,7 @@
 
 ;;; Type conversions.
 
-(defun mucs-ccl-flat-code-to-font-encoding (bytes)
+(defun mccl-font-flat-code-to-font-encoding (bytes)
   (setq mucs-current-type 'font-codepoint)
   (cond ((= bytes 1)
 	 '((r1 = r0)))
@@ -60,11 +62,7 @@
 	 (error "Not support such bytes in font encoding:%S."
 		bytes))))
 
-(defun mucs-ccl-convert-font-encoding (dimension)
-  (if (not (eq mucs-current-type
-	       'font-codepoint))
-      (signal 'mucs-type-mismatch-error
-	      mucs-current-type 'font-codepoint))
+(defun mccl-font-convert-font-encoding-internal (dimension)
   (setq mucs-current-type 'char-1)
   (cond ((= dimension 1)
 	 '((r0 = ((r0 << 16) | r1))))
@@ -74,16 +72,13 @@
 	(t
 	 (error "Unknown dimension:%S." dimension))))
 
-(defun mucs-ccl-font-type-convert (type)
-  (if (not (eq mucs-current-type
-	       'font-codepoint))
-      (signal 'mucs-type-mismatch-error
-	      mucs-current-type 'font-codepoint))
-  (let* ((charset (cdr (assq mccl-font-charset-alist
-			     type)))
-	 (dim (charset-dimension charset)))
-    (setq mucs-current-type 'char-1)
-    (mucs-ccl-convert-font-encoding dim)))
+(defun mccl-font-convert-font-encoding ()
+  (mccl-font-check-conversion-type)
+  (if (not (charsetp mccl-font-current-charset))
+      (error "mccl-font-current-charset is not valid.%S"
+	     mccl-font-current-charset))
+  (mccl-font-convert-font-encoding-internal
+   (charset-dimension mccl-font-current-charset)))
 
 ;
 ; Charset
@@ -109,7 +104,8 @@
       (setq result (cons charset result)))
     (nreverse result)))
 
-(defun mucs-ccl-font-encoder (charset-spec)
+(defun mccl-font-encoder (charset-spec)
+  (mccl-font-check-conversion-type)
   (setq charset-spec
 	(sort (copy-sequence charset-spec)
 	      (lambda (x y) (< (charset-id (car x))
@@ -123,7 +119,8 @@
       (setq id (charset-id (car slot))
 	    charset-spec (cdr charset-spec))
       (while (< i id)
-	(setq charset-clist (cons nil charset-clist)
+	(setq charset-clist (cons (cons nil i)
+				  charset-clist)
 	      i (1+ i)))
       (setq charset-clist (cons slot charset-clist)
 	    i (1+ i)))
@@ -134,20 +131,57 @@
 	,@(mapcar
 	   (lambda (cs)
 	     (cond ((charsetp (car cs))
-		    (setq myo
-			  (mucs-ccl-bind-program (eval (cdr cs))))
-		    (mucs-ccl-myo-append-safe
-		     result-myo
-		     myo)
-		    (mucs-ccl-myo-get-ccl myo))
-;		    (append
-;		     (mucs-ccl-myo-get-ccl myo)
-;		     '((end))))
+		    (let ((mccl-font-current-charset (car cs)))
+		      (setq myo
+			    (mucs-ccl-bind-program (eval (cdr cs))))
+		      (mucs-ccl-myo-append-safe
+		       result-myo
+		       myo)
+		      (mucs-conversion-set-program-marker
+		       (car cs)
+		       (mucs-ccl-myo-get-ccl myo))))
+;		      (append
+;		       (mucs-ccl-myo-get-ccl myo)
+;		       '((end))))
 		   (t
-		    nil)))
+		    (mucs-conversion-set-program-marker
+		     (cdr cs)
+		     (list 'loop nil)))))
 ;		    '((end)))))
 	   charset-clist)))
      result-myo)
     result-myo))
 
-(provide 'mccl-font)
\ No newline at end of file
+(defmacro mccl-font-modify-font-encoder (conv charset spec)
+  (setq conv (eval conv)
+	charset (eval charset)
+	spec (eval spec))
+  (let ((mucs-current-conversion conv)
+	(mccl-font-current-charset charset)
+	mark slot myo ccl-prog)
+    (mccl-font-check-conversion-type)
+    (setq mark charset)
+    (setq slot (mucs-retrieve-marked-conversion-program conv mark))
+    (if (and (null slot)
+	     (setq mark (charset-id charset))
+	     (null (setq slot (mucs-retrieve-marked-conversion-program
+			       conv mark))))
+	(error "Cannot modify conversion:%S for charset:%S"
+	       conv charset))
+    (setq myo (mucs-ccl-bind-program (eval spec))
+	  ccl-prog (mucs-ccl-myo-get-ccl myo))
+    (mucs-substitute-conversion-program
+     conv mark ccl-prog)
+    (setq ccl-prog (list '\` (get conv 'mucs-conversion-program)))
+
+    `(progn
+       ,@(mucs-ccl-make-elisp-preparation-from-myo myo)
+       (put (quote ,conv) 'mucs-conversion-program
+	    ,ccl-prog)
+       (register-ccl-program
+	(quote ,conv)
+	,(ccl-compile (eval ccl-prog)))
+       nil)))
+
+
+(provide 'mccl-font)
