@@ -29,7 +29,7 @@
 (require 'trans-util)
 (require 'ccl)
 
-;;; To cease noisy compiler warnnings.
+;;; To cease noisy compiler warnings.
 (eval-when-compile
   (if (not (boundp 'mucs-version))
       (require 'mucs)))
@@ -694,4 +694,17 @@ If UNIBYTE-P is non-nil, return a unibyte string as a result."
 	   (ccl-compile `(0 ((,reg += ,add)))))))
     sym))
 
+(defun mucs-ccl-nop-program ()
+  (let ((sym
+	 (intern "mucs-ccl-nop")))
+    (if (not (mucs-registered-p 'mucs-ccl-required sym))
+	(progn
+	  (mucs-ccl-register-ccl-program
+	   sym
+	   '((quote ())))
+	  (register-ccl-program
+	   sym
+	   (ccl-compile `(0 ())))))
+    sym))
+
 (provide 'mucs-ccl)
