@@ -45,6 +45,10 @@
 	 'tae-project-all-to-invalid)
 	((eq val t)
 	 'tae-project-all-to-restore)
+	((eq val 'identity)
+	 'tae-project-all-to-identity)
+	((eq val 'lambda)
+	 'tae-project-all-to-lambda)
 	(t
 	 (signal 'wrong-type-argument
 		 (list val)))))
@@ -62,8 +66,12 @@
       (register-code-conversion-map
        sym
        (vector t
-	       (if (eq val 'invalid)
-		   mucs-invalid-code val)
+	       (cond ((eq val 'invalid)
+		      mucs-invalid-code)
+		     ((eq val 'identity)
+		      (mucs-ccl-nop-program))
+		     (t
+		      val))
 	       0 (mucs-max-number)))
       sym)))
 
@@ -538,6 +546,18 @@ after its compilation."
 (defsubst tae-set-tr-elem-all-key (tr-elem value)
   (tae-put-tr-elem-option-alist tr-elem 'all value))
 
+(defsubst tae-tr-elem-not-inverse-p (tr-elem)
+  (cdr (tae-assq-tr-elem-option-alist tr-elem 'not-inverse)))
+
+(defun tae-tr-elem-inverse-p (tr-elem decodep)
+  (and decodep
+       (if (tae-tr-elem-not-inverse-p tr-elem)
+	   (let ((slot (tae-get-tr-elem-type-spec tr-elem)))
+	     (if (eq (car slot) (cdr slot))
+		 nil
+	       (error "not-inverse option flag is valid only if both of the specified types is identical.")))
+	 t)))
+  
 (defun tae-check-and-normalize-range (range-list)
   (let (reduced-range
     	max-rl
@@ -617,62 +637,57 @@ When decodep is non-nil, normalize for decording."
 	       curel alist-copy all-slot)
 	   
 	   ;;; copy alist and remove 'all and invalid slot.
-	   (if decodep
+	   (if (tae-tr-elem-inverse-p tr-elem decodep)
 	       (progn
 		 (setq key-type (cdr (tae-get-tr-elem-type-spec tr-elem))
 		       relative-op (mucs-type-get-relative-op key-type))
 		 (while 
 		     (and (setq curel (car curll))
 			  (cond ((eq (cdr curel) 'all)
-				 (setq all-slot curel)
+				 (setq all-slot (cons (cdr curel)
+						      (car curel)))
 				 nil)
 				;; skip the slot
 				((or (eq (car curel) 'all)
 				     (eq (cdr curel) 'invalid))
 				 (setq curll (cdr curll)))
 				(t
-				 (setq alist-copy (cons (cons (car curel)
-							      (cdr curel))
-							alist-copy))
-				 (setq curll (cdr curll)))))))
-		 (setq key-type (cdr (tae-get-tr-elem-type-spec tr-elem))
-		       relative-op (mucs-type-get-relative-op key-type))
-		 (while 
-		     (and (setq curel (car curll))
-			  (cond ((eq (car curel) 'all)
-				 (setq all-slot curel)
-				 nil)
-				;; skip the slot
-				((or (eq (cdr curel) 'all)
-				     (eq (car curel) 'invalid))
-				 (setq curll (cdr curll)))
-				(t
-				 (setq alist-copy (cons (cons (car curel)
-							      (cdr curel))
+				 (setq alist-copy (cons (cons (cdr curel)
+							      (car curel))
 							alist-copy))
 				 (setq curll (cdr curll)))))))
+	     (setq key-type (cdr (tae-get-tr-elem-type-spec tr-elem))
+		   relative-op (mucs-type-get-relative-op key-type))
+	     (while 
+		 (and (setq curel (car curll))
+		      (cond ((eq (car curel) 'all)
+			     (setq all-slot curel)
+			     nil)
+			    ;; skip the slot
+			    ((or (eq (cdr curel) 'all)
+				 (eq (car curel) 'invalid))
+			     (setq curll (cdr curll)))
+			    (t
+			     (setq alist-copy (cons (cons (car curel)
+							  (cdr curel))
+						    alist-copy))
+			     (setq curll (cdr curll)))))))
 
 	   (tae-set-tr-elem-assoc
 	    tr-elem
 	    (nconc
 	     (sort alist-copy
-		   (if decodep
-		       (lambda (x y) (funcall relative-op
-					      (cdr x) (cdr y)))
-		     (lambda (x y) (funcall relative-op
-					    (car x) (car y)))))))
+		   (lambda (x y) (funcall relative-op
+					  (car x) (car y))))))
 	   (if all-slot
 	       (tae-set-tr-elem-all-key tr-elem
-					(if decodep
-					    (car all-slot)
-					  (cdr all-slot))))
-	   ))
+					(cdr all-slot)))))
 
 	((tae-tr-elem-range-p tr-elem)
 	 (let ((alist (tae-get-tr-elem-range-alist tr-elem))
 	       range-elem i j
 	       newalist)
-	   ;;; We should implement normalization process on range tr-elem.
+	   ;;; We should implement normalization processes on range tr-elem.
 	   ))
 
 	((tae-tr-elem-ccl-p tr-elem)
@@ -708,13 +723,9 @@ When DECODEP is t, reduce for decodeing translation."
 		     (while (setq cur-el2 (car cur-ll2))
 		       (while 
 			   (progn
-			     (cond ((if decodep
-					(funcall relative-op
-						 (cdr cur-el2)
-						 (cdr cur-el1))
-					(funcall relative-op
-						 (car cur-el2)
-						 (car cur-el1)))
+			     (cond ((funcall relative-op
+					     (car cur-el2)
+					     (car cur-el1))
 				    (if (null (car cur-ll1))
 					(setq alist1
 					      (cons cur-el2
@@ -723,13 +734,9 @@ When DECODEP is t, reduce for decodeing translation."
 					      (cons cur-el2
 						    (cdr cur-ll1))))
 				    nil)
-				   ((if decodep
-					(funcall equal-op
-						 (cdr cur-el1)
-						 (cdr cur-el2))
-				      (funcall equal-op
-					       (car cur-el1)
-					       (car cur-el2)))
+				   ((funcall equal-op
+					     (car cur-el1)
+					     (car cur-el2))
 				    nil)
 				   (t
 				    (setq cur-ll1 (cdr cur-ll1))
@@ -824,13 +831,9 @@ When DECODEP is t, reduce for decodeing translation."
 			       cur-ll1 (cons nil alist1))
 			 (while 
 			     (progn
-			       (cond ((if decodep
-					  (funcall equal-op
-						   (cdr cur-el1)
-						   (cdr cur-el2))
-					(funcall equal-op
-						 (car cur-el1)
-						 (car cur-el2)))
+			       (cond ((funcall equal-op
+					       (car cur-el1)
+					       (car cur-el2))
 				      (setq result-alist
 					    (cons cur-el2 result-alist))
 				      nil)
@@ -869,12 +872,13 @@ When DECODEP is t, reduce for decodeing translation."
 TR-ELEMS must be a list that consists of TR-ELEMs
 that must be normalized.
 When DECODEP is t, reduce for decodeing translation."
-  (let ((tr1 (car tr-elems))
-	(tr2 (nth 1 tr-elems))
-	(tr-curl (cdr tr-elems))
-	tr1-all-key
-	tr2-all-key
-	result-list result)
+  (let* ((trs (if decodep (reverse tr-elems) tr-elems))
+	 (tr1 (car trs))
+	 (tr2 (nth 1 trs))
+	 (tr-curl (cdr trs))
+	 tr1-all-key
+	 tr2-all-key
+	 result-list result)
     (while tr2
       (setq tr1-all-key (tae-get-tr-elem-all-key tr1)
 	    tr2-all-key (tae-get-tr-elem-all-key tr2))
@@ -905,11 +909,10 @@ When DECODEP is t, reduce for decodeing translation."
 					   (cons (car cur-el1)
 						 (cdr cur-el2))
 						result-alist))
-				    ;; remove slot at encoding time.
-				    (if (not decodep)
-					(if (null (car cur-ll1))
-					    (setq alist1 (cdr alist1))
-					  (setcdr cur-ll1 (cdr cur-ll1))))
+				    ;; remove the slot that is never used.
+				    (if (null (car cur-ll1))
+					(setq alist1 (cdr alist1))
+				      (setcdr cur-ll1 (cdr cur-ll1)))
 
 				    nil)
 
@@ -919,27 +922,17 @@ When DECODEP is t, reduce for decodeing translation."
 				   )))
 
 		       (if tr1-all-key
-			   (if decodep
-			       (setq result-alist
-				     (cons
-				      (cons tr1-all-key
-					    (car cur-el2))
-				      result-alist))
-			     (if (and (null tr-new-all-key)
-				      (funcall equal-op
-					       tr1-all-key
-					       (car cur-el2)))
-				 (setq tr-new-all-key (cdr cur-el2)))
-				 ))
+			   (if (and (null tr-new-all-key)
+				    (funcall equal-op
+					     tr1-all-key
+					     (car cur-el2)))
+			       (setq tr-new-all-key (cdr cur-el2))))
 		       (setq cur-ll2 (cdr cur-ll2)))
 
 		     ;; operate tr2-all-key
 		     (if tr2-all-key
-			 (if decodep
-		     ;;; At decoding, find out its correspond slot in alist1.
-			     (setq tr-new-all-key
-				   (car (rassq tr2-all-key alist1)))
-		     ;;; At encoding, append rest alist1.
+		     ;;; At encoding, append the rest of alist1.
+			 (progn
 			   (setq cur-ll1 alist1)
 			   (while cur-ll1
 			     (setq result-alist
@@ -1206,17 +1199,11 @@ according to it, rearrange the elements of UNIT."
 		   ;; type-func, conv-func, and to-func are bound by the above `let'.
 		   ;; Owing to Dynamic Binding of Emacs, this function can call
 		   ;; a function bounded by them respectively.
-		 (setq conv-slot (if decodep
-				     (lambda (x)
-				       (cons
-					(funcall type-func
-						 (funcall conv-func (cdr x)))
-					(funcall to-func (car x))))
-				   (lambda (x)
-				     (cons
-				      (funcall type-func
-					       (funcall conv-func (car x)))
-				      (funcall to-func (cdr x))))))
+		 (setq conv-slot (lambda (x)
+				   (cons
+				    (funcall type-func
+					     (funcall conv-func (car x)))
+				    (funcall to-func (cdr x)))))
 		 (list 
 		  (cons 'tables
 			(nconc
@@ -1244,7 +1231,7 @@ according to it, rearrange the elements of UNIT."
 		     (setq ralist (cdr ralist))
 
 		     ;; set st1, st2, end1, end2
-		     (if decodep
+		     (if (tae-tr-elem-inverse-p tr-elem decodep)
 			 (setq slot1 (cdr elem)
 			       slot2 (car elem))
 		       (setq slot1 (car elem)
@@ -1378,7 +1365,6 @@ according to it, rearrange the elements of UNIT."
 	tbls result-mes
 	new-type
 	tr arith)
-    (if decodep (setq translations (reverse translations)))
     (setq tr (car translations)
 	  translations (cdr translations))
     (if tr
