@@ -52,7 +52,7 @@ by calling post-read-conversion and pre-write-conversion functions.")
 
 (defvar un-define-post-read-conversion-charsets-alist
   '((thai-tis620 . thai-post-read-conversion)
-    ;; in-is13194-devanagari-post-read-conversion does not works correctly.
+    ;; in-is13194-devanagari-post-read-conversion does not work correctly.
     ;; I disabled the below line.
     ;; (indian-is13194 . in-is13194-devanagari-post-read-conversion))
     ))
@@ -311,6 +311,13 @@ by calling post-read-conversion and pre-write-conversion functions.")
     (un-define-encode-template
      'unicode-basic-stream-translation-rule
      'emacs-mule 'utf-16-le))))
+ (mucs-define-conversion
+  utf-16-le-no-signature-unix-stream-encoder
+  stream
+  (utf-16-encode-buffer-magnification
+   ((un-define-encode-template
+     'unicode-basic-stream-translation-rule
+     'emacs-mule 'utf-16-le))))
  (mucs-define-conversion
   utf-16-le-unix-stream-decoder
   stream
@@ -327,6 +334,13 @@ by calling post-read-conversion and pre-write-conversion functions.")
     (un-define-encode-template
      'unicode-basic-stream-translation-rule
      'emacs-mule 'utf-16-le-dos))))
+ (mucs-define-conversion
+  utf-16-le-no-signature-dos-stream-encoder
+  stream
+  (utf-16-encode-buffer-magnification
+   ((un-define-encode-template
+     'unicode-basic-stream-translation-rule
+     'emacs-mule 'utf-16-le-dos))))
  (mucs-define-conversion
   utf-16-le-dos-stream-decoder
   stream
@@ -343,6 +357,13 @@ by calling post-read-conversion and pre-write-conversion functions.")
     (un-define-encode-template
      'unicode-mac-line-separator-translation-rule
      'emacs-mule 'utf-16-le))))
+ (mucs-define-conversion
+  utf-16-le-no-signature-mac-stream-encoder
+  stream
+  (utf-16-encode-buffer-magnification
+   ((un-define-encode-template
+     'unicode-mac-line-separator-translation-rule
+     'emacs-mule 'utf-16-le))))
  (mucs-define-conversion
   utf-16-le-mac-stream-decoder
   stream
@@ -360,6 +381,13 @@ by calling post-read-conversion and pre-write-conversion functions.")
     (un-define-encode-template
      'unicode-line-separator-translation-rule
      'emacs-mule 'utf-16-le))))
+ (mucs-define-conversion
+  utf-16-le-no-signature-ul-stream-encoder
+  stream
+  (utf-16-encode-buffer-magnification
+   ((un-define-encode-template
+     'unicode-line-separator-translation-rule
+     'emacs-mule 'utf-16-le))))
  (mucs-define-conversion
   utf-16-le-ul-stream-decoder
   stream
@@ -378,6 +406,13 @@ by calling post-read-conversion and pre-write-conversion functions.")
     (un-define-encode-template
      'unicode-basic-stream-translation-rule
      'emacs-mule 'utf-16-be))))
+ (mucs-define-conversion
+  utf-16-be-no-signature-unix-stream-encoder
+  stream
+  (utf-16-encode-buffer-magnification
+   ((un-define-encode-template
+     'unicode-basic-stream-translation-rule
+     'emacs-mule 'utf-16-be))))
  (mucs-define-conversion
   utf-16-be-unix-stream-decoder
   stream
@@ -394,6 +429,13 @@ by calling post-read-conversion and pre-write-conversion functions.")
     (un-define-encode-template
      'unicode-basic-stream-translation-rule
      'emacs-mule 'utf-16-be-dos))))
+ (mucs-define-conversion
+  utf-16-be-no-signature-dos-stream-encoder
+  stream
+  (utf-16-encode-buffer-magnification
+   ((un-define-encode-template
+     'unicode-basic-stream-translation-rule
+     'emacs-mule 'utf-16-be-dos))))
  (mucs-define-conversion
   utf-16-be-dos-stream-decoder
   stream
@@ -410,6 +452,13 @@ by calling post-read-conversion and pre-write-conversion functions.")
     (un-define-encode-template
      'unicode-mac-line-separator-translation-rule
      'emacs-mule 'utf-16-be))))
+ (mucs-define-conversion
+  utf-16-be-no-signature-mac-stream-encoder
+  stream
+  (utf-16-encode-buffer-magnification
+   ((un-define-encode-template
+     'unicode-mac-line-separator-translation-rule
+     'emacs-mule 'utf-16-be))))
  (mucs-define-conversion
   utf-16-be-mac-stream-decoder
   stream
@@ -427,6 +476,13 @@ by calling post-read-conversion and pre-write-conversion functions.")
     (un-define-encode-template
      'unicode-line-separator-translation-rule
      'emacs-mule 'utf-16-be))))
+ (mucs-define-conversion
+  utf-16-be-no-signature-ul-stream-encoder
+  stream
+  (utf-16-encode-buffer-magnification
+   ((un-define-encode-template
+     'unicode-line-separator-translation-rule
+     'emacs-mule 'utf-16-be))))
  (mucs-define-conversion
   utf-16-be-ul-stream-decoder
   stream
@@ -616,6 +672,38 @@ by calling post-read-conversion and pre-write-conversion functions.")
 	'coding-category-utf-16-le)
       [utf-16-le-unix utf-16-le-dos utf-16-le-mac]))
 
+    (utf-16-le-no-signature
+     (utf-16-le-no-signature-unix
+      ?U "Same as utf-16-le but generate no Unicode signature."
+      utf-16-le-unix-stream-decoder
+      utf-16-le-no-signature-unix-stream-encoder
+      ,(un-define-get-coding-system-alist
+	'coding-category-utf-16-le)
+      unix)
+     (utf-16-le-no-signature-dos
+      ?U "Same as utf-16-le but generate no Unicode signature."
+      utf-16-le-dos-stream-decoder
+      utf-16-le-no-signature-dos-stream-encoder
+      ,(un-define-get-coding-system-alist
+	'coding-category-utf-16-le)
+      dos)
+     (utf-16-le-no-signature-mac
+      ?U "Same as utf-16-le but generate no Unicode signature."
+      utf-16-le-mac-stream-decoder
+      utf-16-le-no-signature-mac-stream-encoder
+      ,(un-define-get-coding-system-alist
+	'coding-category-utf-16-le)
+      mac)
+     (utf-16-le-no-signature
+      ?U "Same as utf-16-le but generate no Unicode signature."
+      utf-16-le-ul-stream-decoder
+      utf-16-le-no-signature-ul-stream-encoder
+      ,(un-define-get-coding-system-alist
+	'coding-category-utf-16-le)
+      [utf-16-le-no-signature-unix
+       utf-16-le-no-signature-dos
+       utf-16-le-no-signature-mac]))
+
     (utf-16-be
      (utf-16-be-unix
       ?U "UTF-16 Big Endian coding system"
@@ -642,6 +730,38 @@ by calling post-read-conversion and pre-write-conversion functions.")
        'coding-category-utf-16-be)
      [utf-16-be-unix utf-16-be-dos utf-16-be-mac]))
 
+    (utf-16-be-no-signature
+     (utf-16-be-no-signature-unix
+      ?U "Same as utf-16-be but generate no Unicode signature."
+      utf-16-be-unix-stream-decoder
+      utf-16-be-no-signature-unix-stream-encoder
+      ,(un-define-get-coding-system-alist
+	'coding-category-utf-16-be)
+      unix)
+    (utf-16-be-no-signature-dos
+     ?U "Same as utf-16-be but generate no Unicode signature."
+     utf-16-be-dos-stream-decoder
+     utf-16-be-no-signature-dos-stream-encoder
+     ,(un-define-get-coding-system-alist
+       'coding-category-utf-16-be)
+     dos)
+    (utf-16-be-no-signature-mac
+     ?U "Same as utf-16-be but generate no Unicode signature."
+     utf-16-be-mac-stream-decoder
+     utf-16-be-no-signature-mac-stream-encoder
+     ,(un-define-get-coding-system-alist
+       'coding-category-utf-16-be)
+     mac)
+    (utf-16-be-no-signature
+     ?U "Same as utf-16-be but generate no Unicode signature."
+     utf-16-be-ul-stream-decoder
+     utf-16-be-no-signature-ul-stream-encoder
+     ,(un-define-get-coding-system-alist
+       'coding-category-utf-16-be)
+     [utf-16-be-no-signature-unix
+      utf-16-be-no-signature-dos
+      utf-16-be-no-signature-mac]))
+
     (utf-7
      (utf-7-unix
       ?7 "UTF-7 coding system.  Output Set O characters directly."
